<?php
namespace FM\Controller;

use FM\Controller\FMCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use FM\Service\DashboardService;
use FM\Traits\TResponse;

class DashboardController extends FMCrud {
   
   use TResponse;
   protected $DashboardService;

    public function __construct($crudService, $environment, DashboardService $DashboardService) {
        parent::__construct($crudService, $environment);
        $this->DashboardService = $DashboardService;
    }

    public function getDadosLimpezaFinalizada(DTO\Request $request, DTO\Response $response) {
      try {
          $data = $this->DashboardService->getDadosLimpezaFinalizada();
          $this->sendResponse($data, 200);
      } catch (Exception $ex) {
          $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
      }
  }

  public function getDadosTarefaAtrasada(DTO\Request $request, DTO\Response $response) {
      try {
        $data = $this->DashboardService->getDadosTarefaAtrasada();
        $this->sendResponse($data, 200);
    } catch (Exception $ex) {
        $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
    }
  }

  public function getDadosTarefaConcluida(DTO\Request $request, DTO\Response $response) {
    try {
        $data = $this->DashboardService->getDadosTarefaConcluida();
        $this->sendResponse($data, 200);
    } catch (Exception $ex) {
        $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
    }
  }

  public function getDadosProgress(DTO\Request $request, DTO\Response $response) {
    try {
      $dadosProgress = $this->DashboardService->getDadosProgress();
      $dtsVal = new DataSet('dados', $dadosProgress);
      $response->addDataSet($dtsVal);
    } catch (Exception $ex) {
      $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
    }
  }

  // ----- Dashboard Supervisor
  public function getDadosUltimasTarefas(DTO\Request $request, DTO\Response $response) {
    try {
        $data = $this->DashboardService->getDadosUltimasTarefas();
        $this->sendResponse($data, 200);
    } catch (Exception $ex) {
        $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
    }
  }

  public function getDadosResumoTarefas(DTO\Request $request, DTO\Response $response) {
      try {
        $data = $this->DashboardService->getDadosResumoTarefas();
        $this->sendResponse($data, 200);
    } catch (Exception $ex) {
        $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
    }
  }

  public function findLocal(DTO\Request $request, DTO\Response $response) {
      try {
        $data = $this->DashboardService->findLocal();
        $this->sendResponse($data, 200);
    } catch (Exception $ex) {
        $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
    }
  }

  public function getDadosChartTarefas(DTO\Request $request, DTO\Response $response) {
    try {
      $dadosTarefaDiaria = $this->DashboardService->getDadosChartTarefas();
      $dtsVal = new DataSet('dados', $dadosTarefaDiaria);
      $response->addDataSet($dtsVal);
    } catch (Exception $ex) {
      $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
    }
  }

  public function getDadosChartNaoConformidades(DTO\Request $request, DTO\Response $response) {

    try {
      $dadosChartNaoConformidades = $this->DashboardService->getDadosChartNaoConformidades();
      $dtsVal = new DataSet('dados', $dadosChartNaoConformidades);
      $response->addDataSet($dtsVal);
    } catch (Exception $ex) {
      $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
    }
  }

  public function getDadosChartProgressoMensal(DTO\Request $request, DTO\Response $response) {

    try {
      $dadosChartProgressoDiario = $this->DashboardService->getDadosChartProgressoMensal();
      $dtsVal = new DataSet('dados', $dadosChartProgressoDiario);
      $response->addDataSet($dtsVal);
    } catch (Exception $ex) {
      $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
    }
  }

  // ------ REQUISIÇÕES DA TELA DE DASHBOARD (GERENCIAL) ----------

  public function setActivities(DTO\Request $request, DTO\Response $response) {

    $activities = $this->DashboardService->setActivities();
    $dtsVal = new DataSet('act', $activities);
    $response->addDataSet($dtsVal);
  }

  public function getLocals(DTO\Request $request, DTO\Response $response) {

    $id = $request->getParameters();

    $locals = $this->DashboardService->getLocals($id);
    $dtsVal = new DataSet('locals', $locals);
    $response->addDataSet($dtsVal);
  }

  public function dataBaseChartBarUnconformities(DTO\Request $request, DTO\Response $response) {
      $parametro = $request->getParameters();

      $dbcb = $this->DashboardService->dataBaseChartBarUnconformities($parametro);
      $dtsVal = new DataSet('dbChartb', $dbcb);
      $response->addDataSet($dtsVal);
  }

  public function dataBaseChartPieClean(DTO\Request $request, DTO\Response $response) {
      $parametro = $request->getParameters();
      // debugApi([$parametro]);
      //dbcb = data base chart bar
      $dbcp = $this->DashboardService->dataBaseChartPieClean($parametro);
      $dtsVal = new DataSet('dbChartp', $dbcp);
      $response->addDataSet($dtsVal);
  }

  public function dataBaseChartPieUnit(DTO\Request $request, DTO\Response $response) {
      $parametro = $request->getParameters();

       // debugApi([$parametro]);

      $dbcp = $this->DashboardService->dataBaseChartPieUnit($parametro);
      $dtsVal = new DataSet('dbChartpu', $dbcp);
      $response->addDataSet($dtsVal);
  }

  public function dataBaseChartBarAvgTime(DTO\Request $request, DTO\Response $response) {
      $parametro = $request->getParameters();

       // debugApi([$parametro]);

      $dbcp = $this->DashboardService->dataBaseChartBarAvgTime($parametro);
      $dtsVal = new DataSet('dbAvgTime', $dbcp);
      $response->addDataSet($dtsVal);
  }

  public function latestReports(DTO\Request $request, DTO\Response $response) {

      $reports = $this->DashboardService->latestReports();
      $dtsVal = new DataSet('reports', $reports);
      $response->addDataSet($dtsVal);
  }
  
  public function percentageTasks(DTO\Request $request, DTO\Response $response) {
      $parametro = $request->getParameters();
      
      $percent = $this->DashboardService->percentageTasks($parametro);
      $dtsVal = new DataSet('percents', $percent);
      $response->addDataSet($dtsVal);
  }
  
  public function tasksCompleted(DTO\Request $request, DTO\Response $response) {
    $parametro = $request->getParameters();
    // debugApi([$parametro]);
    $task = $this->DashboardService->tasksCompleted($parametro);
    $dtsVal = new DataSet('tasks', $task);
    $response->addDataSet($dtsVal);
  }
  
  public function tasksLate(DTO\Request $request, DTO\Response $response) {

      $parametro = $request->getParameters();

      $task = $this->DashboardService->tasksLate($parametro);
      $dtsVal = new DataSet('tasksLate', $task);
      $response->addDataSet($dtsVal);
  }
  
  public function lastTasksPerformed(DTO\Request $request, DTO\Response $response) {
      $parametro = $request->getParameters();

      $task = $this->DashboardService->lastTasksPerformed($parametro);
      $dtsVal = new DataSet('lastTask', $task);
      $response->addDataSet($dtsVal);
  }
  
  public function dataBaseChartPie(DTO\Request $request, DTO\Response $response) {
      $parametro = $request->getParameters();
      
      //dbcp = data base chart pie
      $dbcp = $this->DashboardService->dataBaseChartPie($parametro);
      $dtsVal = new DataSet('dbChartp', $dbcp);
      $response->addDataSet($dtsVal);
  }
  
  public function dataBaseChartBar(DTO\Request $request, DTO\Response $response) {
      $parametro = $request->getParameters();

      //dbcb = data base chart bar
      $dbcb = $this->DashboardService->dataBaseChartBar($parametro);
      $dtsVal = new DataSet('dbChartb', $dbcb);
      $response->addDataSet($dtsVal);
  }
}
?>