<?php

namespace FM\Traits;

use Doctrine\ORM\EntityManager;

trait TQuery {

    /**
     * @return string|null
     */
    private function getPage()
    {
        return !empty($_REQUEST['page']) ? $_REQUEST['page'] : 1;
    }

    /**
     * @return string|null
     */
    private function getLimit()
    {
        return !empty($_REQUEST['limit']) ? $_REQUEST['limit'] : null;
    }

    /**
     * @return string|null
     */
    private function getOrder()
    {
        return !empty($_REQUEST['order']) ? $_REQUEST['order'] : null;
    }

    /**
     * @return array|null
     */
    private function getSearch()
    {
        return !empty($_REQUEST['search']) ? $_REQUEST['search'] : null;
    }

    /**
     * @return array|null
     */
    private function getSearchIn()
    {
        return !empty($_REQUEST['search_in']) ? $_REQUEST['search_in'] : null;
    }

    /**
     * @return array|null
     */
    private function getFind()
    {
        return !empty($_REQUEST['find']) ? $_REQUEST['find'] : null;
    }

    /**
     * @return string|null
     */
    private function getDynamicFilter()
    {
        return !empty($_REQUEST['dynamic_filter']) ? $_REQUEST['dynamic_filter'] : null;
        $column = $_REQUEST['search_in'] ?? null;
        if($column && gettype($_REQUEST['search_in']) == "string"){
            $column = array($_REQUEST['search_in']);
        }
        return $column;
    }

    /**
     * @param $query
     * @param string $pagination
     * @param string $orderBy
     * @param string $search
     * @return string
     */
    private function buildQuery($query, $pagination = '', $orderBy = '', $search = '')
    {
        return "SELECT *
                 FROM ( SELECT USER_QUERY.*, ROWNUM RNUM, COUNT(1) OVER () TOTAL_LINES
                          FROM ( SELECT *
                                   FROM ( $query )
                               $search
                               $orderBy
                                ) USER_QUERY
                      )
                $pagination";
    }

    /**
     * @param $page
     * @param $limit
     * @return string
     */
    public function createPagination ($page, $limit)
    {
        if($page && $limit) {
            $end = $page * $limit;
            $init = $end - $limit + 1;
            return "WHERE RNUM BETWEEN $init AND $end";
        }
        return "";
    }

    /**
     * @param $search
     * @param $searchIn
     * @return string
     *
     */
    public function createSearch ($search, $searchIn)
    {
        if (is_array($searchIn)) {
            $where = 'WHERE ';
            end($searchIn);         // move the internal pointer to the end of the array
            $lastKey = key($searchIn);

            foreach ($searchIn as $index => $column) {
                $where .= "(UPPER(\"$column\") LIKE UPPER('%$search%'))";
                if ($lastKey != $index) {
                    $where .= " OR ";
                }
            }
        } else {
            $where = "WHERE (UPPER(\"$searchIn\") LIKE UPPER('%$search%'))";
        }
        return $where;
    }

    /**
     * @param $dynamicFilter
     * @return string
     */
    private function getDynamicTypeFilter ($filterType): string
    {
        $values = array(
            'CONTAINS'            => 'LIKE',
            'NOT_CONTAINS'        => 'NOT LIKE',
            'EQUALS'              => '=',
            'NOT_EQUALS'          => '<>',
            'GREATER_THAN'        => '>',
            'LESS_THAN'           => '<',
            'GREATER_THAN_EQUALS' => '>=',
            'LESS_THAN_EQUALS'    => '<=',
            'IN'                  => 'IN',
            'NOT_IN'              => 'NOT IN',
            'BETWEEN'             => 'BETWEEN'
        );
        return $values[$filterType];
    }

    /**
     * @param $dynamicFilter
     * @param $search
     * @return string
     */
    public function createDynamicFilter ($dynamicFilter, $search): string
    {
        $dynamicFilter = json_decode(utf8_encode(base64_decode($dynamicFilter)), true);
        // debugApi([$dynamicFilter]);

        $where = '';

        foreach ($dynamicFilter as $column => $filter) {
            foreach ($filter as $filterProps) {
                if(!empty($filterProps['value'])) {
                    $relation  = $filterProps['relation'];
                    $value     = $filterProps['value'];
                    $operation = $this->getDynamicTypeFilter($filterProps['operation']);
                    $where     = empty($search) && empty($where) ? 'WHERE ' : "$where $relation ";
                    $upperIni  = 'UPPER(';
                    $upperEnd  = ')';

                    if(in_array($operation, array('IN', 'NOT IN'))) {
                        if(is_array($value)) {
                            $value = implode("', '", $value);
                        } else {
                            $value = str_replace(";", "', '", $value);
                        }
                    } else if(in_array($operation, array('LIKE', 'NOT LIKE'))) {
                        $value = "%$value%";
                    }

                    if(!in_array($operation, array('LIKE', 'NOT LIKE'))) {
                        $upperIni = $upperEnd = '';
                    }
                    if(in_array($operation, array('BETWEEN'))) {
                        if(is_string($value)) {
                            $betValue = explode(';', $value);
                        } else {
                            $betValue = $value;
                        }
                        if(isset($betValue[0]) && isset($betValue[1])) {
                            $betIni = $betValue[0];
                            $betEnd = $betValue[1];
                            if(is_string($value)) {
                                $value = "'$betIni' AND '$betEnd'";
                            } else {
                                $value  = "TO_DATE('$betIni 00:00:00', 'DD/MM/YYYY HH24:MI:SS') AND TO_DATE('$betEnd 23:59:59', 'DD/MM/YYYY HH24:MI:SS')";
                                $column = "TO_DATE($column, 'DD/MM/YYYY HH24:MI:SS')";
                            }
                        } else {
                            $value = "'0' AND '0'";
                        }
                    } else {
                        $value = "('$value')";
                    }

                    $where .= "($upperIni $column $upperEnd $operation $upperIni $value $upperEnd)";
                }
            }
        }

        return $where;
    }

     private function checkIfStrIsDate($str) {
        return \DateTime::createFromFormat('d/m/Y', $str) !== false || \DateTime::createFromFormat('d/m/Y H:i:s', $str) !== false;
     }

    /**
     * @param $orderBy
     * @return string
     */
    public function createOrderBy ($orderBy)
    {
        $where = "ORDER BY ";

        if (is_array($orderBy)) {
            $where .= implode(", ", str_replace('.', ' ', $orderBy));
        } else {
            $where .= str_replace('.', ' ', $orderBy);
        }

        return strtoupper($where);
    }

    public function createFind($find, $conditions = '')
    {
        if (!empty($find['value']) && !empty($find['column'])) {
            if ($conditions) {
                return "AND {$find['column']} = '{$find['value']}'";
            }
            return "WHERE {$find['column']} = '{$find['value']}'";
        }
        return "";
    }

    /**
     * @param $data
     * @param $page
     * @param $limit
     * @return array
     */
    public function buildResponseData($data, $page = null, $limit = null)
    {
        if (isset($data[0]['TOTAL_LINES'])) {
            $total = $data[0]['TOTAL_LINES'];
        } else if (isset($data['TOTAL_LINES'])) {
            $total = $data['TOTAL_LINES'];
        } else {
            $total = count($data);
        }

        $page = $page ? $page : $this->getPage();
        $limit = $limit ? $limit : ($this->getLimit() ? $this->getLimit() : $total);

        $pagination = array(
            "total" => (int)$total,
            "page"  => (int)$page,
            "limit" => (int)$limit
        );

        return array(
            'data' => $data,
            'pagination' => $pagination
        );
    }

    /**
     * @param $query
     * @param null $page
     * @param null $limit
     * @param null $order
     * @param null $search
     * @param null $searchIn
     * @return string
     */
    public function getFinalQuery($query, $page = null, $limit = null, $order = null, $search = null, $searchIn = null,
                                  $dynamicFilter = null, $find = null)
    {
        $page = $page ? $page : $this->getPage();
        $limit = $limit ? $limit : $this->getLimit();
        $order = $order ? $order : $this->getOrder();
        $search = $search ? $search : $this->getSearch();
        $searchIn = $searchIn ? $searchIn : $this->getSearchIn();
        $find = $find ? $find : $this->getFind();
        $dynamicFilter = $dynamicFilter ? $dynamicFilter : $this->getDynamicFilter();

        $conditions = $search ? $this->createSearch($search, $searchIn) : '';
        $conditions .= $this->createFind($find, $conditions);
        $conditions .= $dynamicFilter ? $this->createDynamicFilter($dynamicFilter, $conditions) : '';

        $pagination = $this->createPagination($page, $limit);
        $order = empty($order) ? '' : $this->createOrderBy($order);

        return $this->buildQuery($query, $pagination, $order, $conditions);
    }

    /**
     * @param EntityManager $entityManager
     * @param string $finalQuery
     * @param array $params
     * @param array $types
     * @param null $page
     * @param null $limit
     * @return array
     */
    public function executeQueryAndBuildResponseData(EntityManager $entityManager, $finalQuery, $params = [],
                                                     $types = [], $page = null, $limit = null)
    {
        $data = $entityManager->getConnection()->fetchAll($finalQuery, $params, $types);
        return $this->buildResponseData($data, $page, $limit);
    }

    /**
     * @param EntityManager $entityManager
     * @param string $query
     * @param array $params
     * @param array $types
     * @param null $page
     * @param null $limit
     * @param null $order
     * @return array
     */
    public function createAndExecuteQueryAndBuildResponseData(EntityManager $entityManager, $query, $params = [],
                                                              $types = [], $page = null, $limit = null, $order = null)
    {
        $finalQuery = $this->getFinalQuery($query, $page, $limit, $order);
        return $this->executeQueryAndBuildResponseData($entityManager, $finalQuery, $params, $types, $page, $limit);
    }
}
